/*
dtmflogd        A program to log digits captured by a serial dtmf decoder
Copyright (C) 2003  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <termios.h>
#include <signal.h>
#include <sys/types.h>

#include "ttyS.h"
#include "dtmfd.h"
#include "logging.h"
#include "cfg.h"
#include "daemonize.h"

static int port_fd=-1;
static struct termios oldtio;

void signal_handler(int signum);



//#define DEAMON_MODE

int main(int argc, char **argv)
{
	unsigned long attributes=0;
	int res;

	signal(SIGHUP, &signal_handler);
	signal(SIGINT, &signal_handler);
	signal(SIGTERM, &signal_handler);
	signal(SIGQUIT, &signal_handler);
	
	/* Process command line args */
	while(1)
	{
		res = getopt(argc, argv, "Dhd:s:m:f:");
		if (res<0) break;

		switch(res)
		{
			case 'D': // Disable deamonize
				g_cfg_daemonize=0;
				break;
			case 'd': // set serial port
				g_cfg_ttyS_device=optarg;
				break;
			case 's': // set speed
				g_cfg_ttyS_speed=optarg;
				break;
			case 'm': // set mode
				g_cfg_ttyS_mode=optarg;
				break;
			case 'f': // log file
				g_cfg_log_file=optarg;			
				break;
			case 'h':
	printf("Usage: ./dtmflogd [OPTION]...\n");
	printf("Used to log digits received from a serial\n");
	printf("dtmf decoder.\n\n");
	
	printf("  -D              Disable daemon mode (Dont go in background)\n");
	printf("  -d              Specify the serial device to use (default /dev/ttyS0)\n");
	printf("  -s              Set serial port speed (default 9600)\n");
	printf("  -m              Set serial port mode (default 8N1)\n");
	printf("  -f              Set log file (default ./log.txt)\n");
	printf("  -h              Print this screen\n");
	printf("\n");
	printf("dtmflogd comes with ABSOLUTELY NO WARRANTY; for details\n");
	printf("see LICENSE. This is free software, and you are welcome\n");
	printf("to redistribute it under certain conditions; see COPYING\n");
	printf("for details.\n\n");
	printf("Author: Raphael Assenat <raph@raphnet.net>\n");
		
				return 0;
				break;
		}
	}

	
	attributes = getBaudRateBits(atoi(g_cfg_ttyS_speed));
	getModeBits(g_cfg_ttyS_mode, &attributes);
	
	printf("dtmflogd starting\n");
	printf("Using %s at %d bps %s\n", g_cfg_ttyS_device,
						atoi(g_cfg_ttyS_speed),
							g_cfg_ttyS_mode);

	port_fd = openport(g_cfg_ttyS_device, &oldtio, attributes);
	if (port_fd<0) {
		return -1;
	}

	printf("Using log file: %s\n", g_cfg_log_file);
	res = init_logging(g_cfg_log_file);
	if (res<0) {
		fprintf(stderr, "Aborting\n");
		shutdown();
	}
	
	if (g_cfg_daemonize) {
		res = daemonize();
		if (res==0) {
			exit(0); // parent
		}
		if (res<0) {
			fprintf(stderr, "Could not go into daemon mode\n");
			shutdown_logging();
			shutdown();
		}
	}
	
	start_logging(port_fd);	

	shutdown_logging();
	
	shutdown();
	return 0;
}

void shutdown(void)
{
	if (port_fd!=-1)
	{
		/* retore settings */
		tcsetattr(port_fd, TCSANOW, &oldtio);

		close(port_fd);
	}
	exit(0);
}

void signal_handler(int signum)
{
	static int term_retries=0;
	
	switch(signum)
	{
		case SIGINT:
		case SIGQUIT:
		case SIGTERM:
			if (term_retries==0) {
				printf("stopping...\n");
				mustrun=0;
				term_retries++;
			} else {
				printf("Disgracefully killing ourself\n");
				kill(getpid(), SIGKILL);
			}			
			break;
		case SIGHUP:
	}
}

