/*
dtmflogd        A program to log digits captured by a serial dtmf decoder
Copyright (C) 2003  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "pnum.h"

int pnum_remove_function(const char *src_str, char *function, char *number)
{
	int len=strlen(src_str);
	if (len<3) { return -1; }
	
	if (src_str[0]=='*')
	{
		function[0]=src_str[0];
		function[1]=src_str[1];
		function[2]=src_str[2];
		function[3]=0;

		strcpy(number, &src_str[3]);
		return 1;
	}
	strcpy(number, src_str);
	
	return 0;
}

char *pnum_toString(char *dst_buf, PhoneNumber *pnumber)
{
	snprintf(dst_buf, PNUM_MAX_LEN, "%s%03d-%03d-%04d", 
			pnumber->isLongDistance?"1-":"",
			pnumber->npa, pnumber->co,
			pnumber->number);
	return dst_buf;
}

char *pnum_getFunctionName(char *fn)
{
	int i=0;
	if (strlen(fn)<3) { return NULL; }
	
	i = atoi(&fn[1]);
	
	switch(i)
	{
		case 11: return "Peace & quiet (Call privacy)";	 
		case 57: return "Caller retrace";
		case 60: return "Screen out unwanted numbers (Call privacy)";
		case 67: return "Caller-id blocking";
		case 66: return "Busy call return";
		case 69: return "Last call return";			 
		case 70: return "Call waiting";
		case 71: return "Three-Way calling";
		case 72: return "Enable call forwarding";
		case 73: return "Disable call forwarding";		 
		case 98: return "Call answer";
			
	}
	return NULL;	
}

int pnum_split_and_expand(char *number, PhoneNumber *pnumber)
{
	int len;
	char tmp[10];

	len=strlen(number);

	switch(len)
	{
		case LOCAL_NUMBER_LEN:
			pnumber->npa = AREA_CODE;			
			memcpy(tmp, &number[0], 3); tmp[3]=0;
			pnumber->co = atoi(tmp);
			memcpy(tmp, &number[3], 4); tmp[4]=0;
			pnumber->number = atoi(tmp);
			pnumber->isLongDistance=0;
			break;
		case AREA_NUMBER_LEN:
			memcpy(tmp, &number[0], 3); tmp[3]=0;
			pnumber->npa = atoi(tmp);
			memcpy(tmp, &number[3], 3); tmp[3]=0;
			pnumber->co = atoi(tmp);
			memcpy(tmp, &number[6], 4); tmp[4]=0;
			pnumber->number = atoi(tmp);
			pnumber->isLongDistance=0;
			break;
		case LONG_DISTANCE_NUMBER_LEN:
			memcpy(tmp, &number[1], 3); tmp[3]=0;
			pnumber->npa = atoi(tmp);
			memcpy(tmp, &number[4], 3); tmp[3]=0;
			pnumber->co = atoi(tmp);
			memcpy(tmp, &number[7], 4); tmp[4]=0;
			pnumber->number = atoi(tmp);
			pnumber->isLongDistance=1;
			break;
		default:
			return -1; // malformed number or preceding *??
	}

	return 0;
}

